
# FIXME: Document.
get_top_true_mgs <- function(mgs,
                             n = "all",
                             direction = "both",
                             sort_by_score = "mean_score",
                             de_value_cutoff = 1,
                             up_mean_filter = 0.1
                             ) {

  # "both" is a derived not given label.
  if (direction == "both") {
    mgs <- mgs[mgs$direction == "up" | mgs$direction == "down", ]
  } else {
    mgs <- mgs[mgs$direction == direction, ]
  }


  if (direction == "up") {
    mgs <- mgs[mgs$de_value > de_value_cutoff, ]

    mgs <- mgs[mgs$gene_mean > up_mean_filter, ]

  } else if (direction == "down") {
    mgs <- mgs[mgs$de_value < de_value_cutoff, ]
  } else if (direction == "both") {
    mgs <- mgs[mgs$de_value < de_value_cutoff | mgs$de_value > de_value_cutoff, ]
  } else{
    mgs <- mgs
  }

  mgs <- mgs[order(abs(mgs[[sort_by_score]]), decreasing = TRUE), ]

  if (n == "all") {
    n <- nrow(mgs)
  }

  mgs[1:n, ]
}

#' Select the top n marker genes from a marker gene selection method's output
#'
#' This function is *extremely important* for the calculation of all the
#' simulation results.
#'
#' @param mgs The output of the marker gene selection method in standard form.
#' @param n The number of top marker genes to select. A single integer.
#' @param direction
#'
#' @return A reduced marker genes object.
#'
get_top_sel_mgs <- function(mgs, n, direction = "both") {


  # FIXME: Ideally we would like to AND here except the default value of 0
  # means that will not be sensible.
  if (direction == "up") {
    up_inds <- mgs$raw_statistic > 0 |
      mgs$scaled_statistic > 0 |
      mgs$log_fc > 0
    mgs <- mgs[up_inds, ]
  } else if (direction == "down") {
    down_inds <- mgs$raw_statistic < 0 |
      mgs$scaled_statistic < 0 |
      mgs$log_fc < 0
    mgs <- mgs[down_inds, ]
  } else {
    mgs
  }

  mgs[1:n, ]
}

#' Calculate the true positive rate
#'
#' @param found The marker genes found.
#' @param true The true marker genes.
#'
#' @return The true positive rate
#'
calculate_tpr <- function(found, true) {
  stopifnot(is.vector(found) && is.vector(true))
  n_tp <- length(base::intersect(found, true))
  n <- length(true)
  n_tp / n
}


retrieve_real_data_parameters <- function() {

  real_data_folder <- here::here("results", "real_data")
  method_pars_folder <- here::here("config", "method_pars")

  paths <- list.files(real_data_folder, full.names = TRUE)
  files <- basename(paths)

  res_tibs <- list()
  for (i in seq_along(paths)) {

    path <- paths[[i]]
    file <- files[[i]]

    # Remove .rds from end of file.
    file <- substr(file, 1, nchar(file) - 4)

    # Split file on the "-".
    split_file <- strsplit(file, "-")[[1]]

    data_id <- split_file[[1]]
    method_id <- split_file[[2]]

    method_id_path <- paste0(file.path(method_pars_folder, method_id), ".json")
    method_pars <- jsonlite::fromJSON(method_id_path)

    tib <- tibble::as_tibble(method_pars)
    tib <- dplyr::mutate(
      tib,
      data_id = data_id,
      pars = method_id,
      full_filename = path
    )
    tib <- dplyr::rename(tib, method_name = file_name)

    res_tibs[[i]] <- tib
  }

  out <- dplyr::bind_rows(res_tibs)
  out
}


retrive_simulation_parameters <- function() {

  config <- yaml::read_yaml(here::here("config.yaml"))

  res_paths <- list.files(here::here("results", "sim_data"), full.names = TRUE)
  res_files <- basename(res_paths)

  res_tibs <- list()
  for (i in seq_along(res_files)) {

    path <- res_paths[[i]]
    file <- res_files[[i]]
    full_filename <- file

    # Remove .rds from end of file.
    file <- substr(file, 1, nchar(file) - 4)

    split_file <- strsplit(file, "-")[[1]]

    sim_id <- split_file[[1]]
    data_id <- split_file[[2]]
    method_id <- split_file[[3]]

    method_par_path <- paste0(here::here(config$method_pars_folder, method_id), ".json")
    sim_par_path <- paste0(here::here(config$sim_pars_folder, sim_id), ".json")

    method_pars <- jsonlite::fromJSON(method_par_path)
    sim_pars_raw <- jsonlite::fromJSON(sim_par_path)

    # This converts 'true' vectors in the list into sublists.
    sim_pars <- list()
    for (name in names(sim_pars_raw)) {
      if (length(sim_pars_raw[[name]]) > 1) {
        sim_pars[[name]] <- list(sim_pars_raw[[name]])
      } else {
        sim_pars[[name]] <- sim_pars_raw[[name]]
      }
    }

    sim_pars_tib <- tibble::as_tibble(sim_pars)
    method_pars_tib <- tibble::as_tibble(method_pars)
    data_id_tib <- tibble::tibble(data_id = data_id)
    # Add the full list of parameters for backwards compatibility.
    method_pars_tib <- tibble(method_pars_tib, pars = method_id)

    sim_pars_tib <- dplyr::rename(
      sim_pars_tib,
      sim_label = label,
      sim_name = file_name
    )
    method_pars_tib <- dplyr::rename(method_pars_tib, method_name = file_name)

    all_pars_tib <- dplyr::bind_cols(sim_pars_tib, method_pars_tib, data_id_tib)
    all_pars_tib <- tibble::add_column(all_pars_tib, file_name = full_filename)

    all_pars_tib <- mutate(all_pars_tib, full_filename = path)

    res_tibs[[i]] <- all_pars_tib
  }

  out <- dplyr::bind_rows(res_tibs)

  out
}
